@@ -3,7 +3,7 @@
 
   AUTHORS:
 
-  mario.kleiner at tuebingen.mpg.de  mk
+  mario.kleiner.de@gmail.com  mk
 
   PLATFORMS:    All
 
@@ -33,15 +33,15 @@
  * Routines for debugging the window server and application drawing.
  *
  * Copyright (C) 2007-2008 Alacatia Labs
- * 
+ *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
  * arising from the use of this software.
- * 
+ *
  * Permission is granted to anyone to use this software for any purpose,
  * including commercial applications, and to alter it and redistribute it
  * freely, subject to the following restrictions:
- * 
+ *
  * 1. The origin of this software must not be misrepresented; you must not
  *    claim that you wrote the original software. If you use this software
  *    in a product, an acknowledgment in the product documentation would be
@@ -49,64 +49,64 @@
  * 2. Altered source versions must be plainly marked as such, and must not be
  *    misrepresented as being the original software.
  * 3. This notice may not be removed or altered from any source distribution.
- * 
+ *
  * Joe Ranieri joe@alacatia.com
  *
  *
  */
- 
+
 typedef enum {
 	/*! Clears all flags. */
 	kCGSDebugOptionNone = 0,
-	
+
 	/*! All screen updates are flashed in yellow. Regions under a DisableUpdate are flashed in orange. Regions that are hardware accellerated are painted green. */
 	kCGSDebugOptionFlashScreenUpdates = 0x4,
-	
+
 	/*! Colors windows green if they are accellerated, otherwise red. Doesn't cause things to refresh properly - leaves excess rects cluttering the screen. */
 	kCGSDebugOptionColorByAccelleration = 0x20,
-	
+
 	/*! Disables shadows on all windows. */
 	kCGSDebugOptionNoShadows = 0x4000,
-	
+
 	/*! Setting this disables the pause after a flash when using FlashScreenUpdates or FlashIdenticalUpdates. */
 	kCGSDebugOptionNoDelayAfterFlash = 0x20000,
-	
+
 	/*! Flushes the contents to the screen after every drawing operation. */
 	kCGSDebugOptionAutoflushDrawing = 0x40000,
-	
+
 	/*! Highlights mouse tracking areas. Doesn't cause things to refresh correctly - leaves excess rectangles cluttering the screen. */
 	kCGSDebugOptionShowMouseTrackingAreas = 0x100000,
-	
+
 	/*! Flashes identical updates in red. */
 	kCGSDebugOptionFlashIdenticalUpdates = 0x4000000,
-	
+
 	/*! Dumps a list of windows to /tmp/WindowServer.winfo.out. This is what Quartz Debug uses to get the window list. */
 	kCGSDebugOptionDumpWindowListToFile = 0x80000001,
-	
+
 	/*! Dumps a list of connections to /tmp/WindowServer.cinfo.out. */
 	kCGSDebugOptionDumpConnectionListToFile = 0x80000002,
-	
+
 	/*! Dumps a very verbose debug log of the WindowServer to /tmp/CGLog_WinServer_<PID>. */
 	kCGSDebugOptionVerboseLogging = 0x80000006,
 
 	/*! Dumps a very verbose debug log of all processes to /tmp/CGLog_<NAME>_<PID>. */
 	kCGSDebugOptionVerboseLoggingAllApps = 0x80000007,
-	
+
 	/*! Dumps a list of hotkeys to /tmp/WindowServer.keyinfo.out. */
 	kCGSDebugOptionDumpHotKeyListToFile = 0x8000000E,
-	
+
 	/*! Dumps information about OpenGL extensions, etc to /tmp/WindowServer.glinfo.out. */
 	kCGSDebugOptionDumpOpenGLInfoToFile = 0x80000013,
-	
+
 	/*! Dumps a list of shadows to /tmp/WindowServer.shinfo.out. */
 	kCGSDebugOptionDumpShadowListToFile = 0x80000014,
-	
+
 	/*! Leopard: Dumps information about caches to `/tmp/WindowServer.scinfo.out`. */
 	kCGSDebugOptionDumpCacheInformationToFile = 0x80000015,
-	
+
 	/*! Leopard: Purges some sort of cache - most likely the same caches dummped with `kCGSDebugOptionDumpCacheInformationToFile`. */
 	kCGSDebugOptionPurgeCaches = 0x80000016,
-	
+
 	/*! Leopard: Dumps a list of windows to `/tmp/WindowServer.winfo.plist`. This is what Quartz Debug on 10.5 uses to get the window list. */
 	kCGSDebugOptionDumpWindowListToPlist = 0x80000017,
 
@@ -114,22 +114,22 @@ typedef enum {
 	kCGSDebugOptionEnableSurfacePurging = 0x8000001B,
 
 	// Leopard: 0x8000001C - invalid
-	
+
 	/*! Leopard: DOCUMENTATION PENDING */
 	kCGSDebugOptionDisableSurfacePurging = 0x8000001D,
-	
+
 	/*! Leopard: Dumps information about an application's resource usage to `/tmp/CGResources_<NAME>_<PID>`. */
 	kCGSDebugOptionDumpResourceUsageToFiles = 0x80000020,
-	
+
 	// Leopard: 0x80000022 - something about QuartzGL?
-	
+
 	// Leopard: Returns the magic mirror to its normal mode. The magic mirror is what the Dock uses to draw the screen reflection. For more information, see `CGSSetMagicMirror`. */
 	kCGSDebugOptionSetMagicMirrorModeNormal = 0x80000023,
-	
+
 	/*! Leopard: Disables the magic mirror. It still appears but draws black instead of a reflection. */
 	kCGSDebugOptionSetMagicMirrorModeDisabled = 0x80000024,
 } CGSDebugOption;
- 
+
 typedef int CGSConnectionID;
 extern CGSConnectionID _CGSDefaultConnection(void);
 extern CGError CGSGetPerformanceData(CGSConnectionID cid, float *outFPS, float *unk, float *unk2, float *unk3);
@@ -139,7 +139,7 @@ extern int CGSGetDebugOptions(int *outCurrentOptions);
 #endif
 
 static char useString[] = "info = Screen('GetWindowInfo', windowPtr [, infoType=0] [, auxArg1]);";
-static char synopsisString[] = 
+static char synopsisString[] =
     "Returns a struct with miscellaneous info for the specified onscreen window.\n\n"
     "\"windowPtr\" is the handle of the onscreen window for which info should be returned.\n\n"
     "\"infoType\" If left out or set to zero, all available information for the 'windowPtr' is returned.\n\n"
@@ -173,6 +173,7 @@ static char synopsisString[] =
     "An 'infoType' of 7 does return the same information as the default 'infoType' 0, but does not "
     "set the window 'windowPtr' as drawing target, does not activate its OpenGL context and only "
     "returns information that is safe to return without setting the window as drawing target.\n\n"
+    "An 'infoType' of 8 returns 1 if the X-Screens primary gpu uses the modesetting-ddx under Linux.\n\n"
     "The info struct contains all kinds of information. Just check its output to see what "
     "is returned. Most of this info is not interesting for normal users, mostly provided "
     "for internal use by M-Files belonging to Psychtoolbox itself, e.g., display tests.\n\n"
@@ -209,6 +210,7 @@ static char synopsisString[] =
     "R100 = Very old ATI GPUs, R300 = GPU's roughly starting at Radeon 9000, R500 = Radeon X1000 or later, R600 = Radeon HD2000 or later.\n"
     "NV10 = Very old NVidia GPUs, NV30 = NV30 or later, NV40 = Geforce6000/7000 or later, G80 = Geforce8000 or later.\n"
     "An empty GPUCoreId string means a different, unspecified core.\n\n"
+    "DisplayCoreId: Vendor of the display engine / display gpu, NVidia, AMD, Intel, or same as 'GPUCoreId'. May differ from 'GPUCoreId' in hybrid graphics laptops.\n"
     "BitsPerColorComponent: Effective color depths of window/framebuffer in bits per color channel component (bpc).\n"
     "GLSupportsFBOUpToBpc: 0 = No support for framebuffer objects. Otherwise maximum supported bpc (8, 16, 32).\n"
     "GLSupportsBlendingUpToBpc: Maximum supported bpc for hardware accelerated framebuffer blending (alpha blending).\n"
@@ -216,20 +218,20 @@ static char synopsisString[] =
     "GLSupportsFilteringUpToBpc: Maximum supported bpc for hardware accelerated linear filtering of textures (8, 16, 32).\n"
     "GLSupportsPrecisionColors: 1 = Hardware can be fully trusted to rasterize perfect 32 bpc colors in floating point color mode "
     "without special support of PTB. 0 = Needs special (slower) support from PTB to work.\n"
-    "GLSupportsFP32Shading: 1 = All internal caculations of the GPU are done with IEEE single precision 32 bit floating point, i.e., "
+    "GLSupportsFP32Shading: 1 = All internal calculations of the GPU are done with IEEE single precision 32 bit floating point, i.e., "
     "very accurate.\n"
     "GPUMinorType: Numeric vendor specific gpu id. Chip family on NVidia, display engine revision on AMD atm. -1 if unknown. Subject to change without notice!\n\n";
 
 static char seeAlsoString[] = "OpenWindow, Flip, NominalFrameRate";
 
-PsychError SCREENGetWindowInfo(void) 
+PsychError SCREENGetWindowInfo(void)
 {
     const char *FieldNames[]={ "Beamposition", "LastVBLTimeOfFlip", "LastVBLTime", "VBLCount", "TimeAtSwapRequest", "TimePostSwapRequest", "RawSwapTimeOfFlip",
                                 "VBLTimePostFlip", "OSSwapTimestamp", "GPULastFrameRenderTime", "StereoMode", "ImagingMode", "MultiSampling", "MissedDeadlines", "FlipCount", "StereoDrawBuffer",
                                 "GuesstimatedMemoryUsageMB", "VBLStartline", "VBLEndline", "VideoRefreshFromBeamposition", "GLVendor", "GLRenderer", "GLVersion", "GPUCoreId", "GPUMinorType",
-                                "GLSupportsFBOUpToBpc", "GLSupportsBlendingUpToBpc", "GLSupportsTexturesUpToBpc", "GLSupportsFilteringUpToBpc", "GLSupportsPrecisionColors",
+                                "DisplayCoreId", "GLSupportsFBOUpToBpc", "GLSupportsBlendingUpToBpc", "GLSupportsTexturesUpToBpc", "GLSupportsFilteringUpToBpc", "GLSupportsPrecisionColors",
                                 "GLSupportsFP32Shading", "BitsPerColorComponent", "IsFullscreen", "SpecialFlags", "SwapGroup", "SwapBarrier" };
-    const int fieldCount = 36;
+    const int fieldCount = 37;
     PsychGenericScriptType *s;
 
     PsychWindowRecordType *windowRecord;
@@ -241,19 +243,19 @@ PsychError SCREENGetWindowInfo(void)
     psych_bool onscreen;
     int queryState;
     unsigned int gpuTimeElapsed;
-    int gpuMinorType;
+    int gpuMaintype, gpuMinorType;
 
-    //all subfunctions should have these two lines.  
+    //all subfunctions should have these two lines.
     PsychPushHelp(useString, synopsisString, seeAlsoString);
     if(PsychIsGiveHelp()){PsychGiveHelp();return(PsychError_none);};
 
     PsychErrorExit(PsychCapNumInputArgs(5));     //The maximum number of inputs
-    PsychErrorExit(PsychRequireNumInputArgs(1)); //The required number of inputs	
+    PsychErrorExit(PsychRequireNumInputArgs(1)); //The required number of inputs
     PsychErrorExit(PsychCapNumOutputArgs(1));    //The maximum number of outputs
 
     // Query infoType flag: Defaults to zero.
     PsychCopyInIntegerArg(2, FALSE, &infoType);
-    if (infoType < 0 || infoType > 7) PsychErrorExitMsg(PsychError_user, "Invalid 'infoType' argument specified! Valid are 0, 1, 2, 3, 4, 5, 6, 7.");
+    if (infoType < 0 || infoType > 8) PsychErrorExitMsg(PsychError_user, "Invalid 'infoType' argument specified! Valid are 0, 1, 2, 3, 4, 5, 6, 7, 8.");
 
     // Windowserver info requested?
     if (infoType == 2 || infoType == 3) {
@@ -415,6 +417,14 @@ PsychError SCREENGetWindowInfo(void)
         // Report if a query was pending:
         PsychCopyOutDoubleArg(1, FALSE, (windowRecord->gpuRenderTimeQuery) ? 1 : 0);
     }
+    else if (infoType == 8) {
+        // Modesetting ddx on Linux X11 in use?
+        #if PSYCH_SYSTEM == PSYCH_LINUX
+            PsychCopyOutDoubleArg(1, FALSE, PsychOSX11ScreenUsesModesettingDDX(windowRecord->screenNumber));
+        #else
+            PsychErrorExitMsg(PsychError_unimplemented, "infoType 8 query is only supported on Linux.");
+        #endif
+    }
     else {
         // Set OpenGL context (always needed) and drawing target, as setting
         // our windowRecord as a drawingtarget is an expected side-effect of
@@ -562,9 +572,10 @@ PsychError SCREENGetWindowInfo(void)
             PsychSetStructArrayDoubleElement("GLSupportsFP32Shading", 0, 1, s);
         } else PsychSetStructArrayDoubleElement("GLSupportsFP32Shading", 0, 0, s);
 
-        // Try to get and return vendor specific gpuMinorType:
-        // On AMD this is the DCE display engine revision, on NVidia the chip family name:
-        if (!PsychGetGPUSpecs(windowRecord->screenNumber, NULL, &gpuMinorType, NULL, NULL)) {
+        // Try to get and return vendor specific gpuMaintype + gpuMinorType:
+        // On AMD gpuMinorType is the DCE display engine revision, on NVidia the chip family name:
+        gpuMaintype = 0;
+        if (!PsychGetGPUSpecs(windowRecord->screenNumber, &gpuMaintype, &gpuMinorType, NULL, NULL)) {
             // Query failed or unsupported:
             PsychSetStructArrayDoubleElement("GPUMinorType", 0, -1, s);
         }
@@ -572,6 +583,21 @@ PsychError SCREENGetWindowInfo(void)
             PsychSetStructArrayDoubleElement("GPUMinorType", 0, (double) gpuMinorType, s);
         }
 
+        switch (gpuMaintype) {
+            case kPsychGeForce:
+                PsychSetStructArrayStringElement("DisplayCoreId", 0, "NVidia", s);
+                break;
+            case kPsychRadeon:
+                PsychSetStructArrayStringElement("DisplayCoreId", 0, "AMD", s);
+                break;
+            case kPsychIntelIGP:
+                PsychSetStructArrayStringElement("DisplayCoreId", 0, "Intel", s);
+                break;
+            default:
+                PsychSetStructArrayStringElement("DisplayCoreId", 0, windowRecord->gpuCoreId, s);
+                break;
+        }
+
         if (infoType != 7) {
             // Renderer information: This comes last, and would fail if async flips
             // are active, because it needs PsychSetDrawingTarget, which in turn needs async
