@@ -35,6 +35,13 @@ function [win, winRect] = BitsPlusPlus(cmd, arg, dummy, varargin)
 % Bits++.
 %
 %
+% BitsPlusPlus('UseFE1StereoGoggles', window, enable [, shutterState=0]);
+% If 'enable' is 1, enable use of the CRS FE1 stereo goggles connected
+% to the Bits display device. If 'enable' is 0 then disable use of the
+% goggles again and put them in a resting state with either both shutters
+% open ('shutterState = 0') or closed ('shutterState = 1').
+%
+%
 % Schedule a Bits++ DIO command for execution on next Screen('Flip'):
 %
 % BitsPlusPlus('DIOCommand', window, repetitions, mask, data, command [, xpos, ypos]);
@@ -355,6 +362,16 @@ function [win, winRect] = BitsPlusPlus(cmd, arg, dummy, varargin)
 % 12.01.2013 Make compatible with PTB panelfitter. (MK)
 % 13.03.2013 Make compatible with CRS Bits# video display system. (MK)
 % 15.04.2013 Add mode = BitsPlusPlus('GetColorConversionMode', win); (MK)
+% 24.06.2016 Improved diagnostics. Now need validation from BitsPlusImagingPipelineTest
+%            and BitsPlusIdentityClutTest, instead of only BitsPlusImagingPipelineTest.
+%            Now also triggers revalidation if OS kernel version/release changes, as
+%            things like dithering or output color depth control are usually located in
+%            the kernel level display drivers, so new kernel can imply new bugs.
+% 01.05.2017 Replace evalin('base', ...) call by direct call in PsychDataPixx clut updates,
+%            as a small performance optimization. (MK)
+%            Now possible due to ability for Screen() to assign imaging pipeline variables
+%            to callers workspace. This requires Octave 3.8+ or Matlab R2012a, which is our
+%            supported minimum requirement for Psychtoolbox 3.0.14.
 
 global GL;
 
@@ -926,7 +943,8 @@ if strcmpi(cmd, 'StoreValidation')
     % Enforce use of this routine without verification of correct function
     % of the imaging pipeline. This is used by the correctness test itself
     % in order to be able to run the validation.
-    ValidateBitsPlusImaging(arg, 1, devname);
+    validationType = dummy;
+    ValidateBitsPlusImaging(arg, 1, devname, validationType);
     return;
 end
 
@@ -1063,18 +1081,13 @@ if strcmpi(cmd, 'OpenWindowBits++')
     end
     
     if targetdevicetype == 1
-        % We need this weird evalin('base', ...); wrapper so the
-        % function gets called from the base-workspace, where the
-        % IMAGINGPIPE_GAMMATABLE variable is defined. We can only
-        % define it there reliably due to incompatibilities between
-        % Matlab and Octave in variable assignment inside Screen() :-(
-        rclutcmd = 'evalin(''base'', ''PsychDataPixx(1, IMAGINGPIPE_GAMMATABLE);'');';
+        rclutcmd = 'PsychDataPixx(1, IMAGINGPIPE_GAMMATABLE);';
         Screen('HookFunction', win, 'PrependMFunction', 'LeftFinalizerBlitChain', 'Upload new clut into DataPixx callback', rclutcmd);
     end
     
     Screen('HookFunction', win, 'Enable', 'LeftFinalizerBlitChain');
 
-    if (~isempty(stereomode) && stereomode == 1)
+    if ~isempty(stereomode) && ismember(stereomode, [1, 11])
         % This is only needed on quad-buffered stereo contexts for Bits+.
         % Enable CLUT updates via T-Lock on right stereo buffer as well:
 
@@ -1093,25 +1106,23 @@ if strcmpi(cmd, 'OpenWindowBits++')
         % Setup finalizer callback for DIO T-Lock updates:
         tlockhandle = SetupDIOFinalizer(win, stereomode);
     end
-    
+
     % Load an identity CLUT into the Bits++ to start with:
     linear_lut =  repmat(linspace(0, 1, 256)', 1, 3);
     Screen('LoadNormalizedGammaTable', win, linear_lut, 2);
     
     % Check validation:
     if ~validated
-        % MK: Actually, don't! Validation code doesn't check/validate
-        % anything in Bits++ mode, so this is pointless... Leave it here
-        % for documentation.
-        % ValidateBitsPlusImaging(win, 0, devname);
+        % Only validate output display pipeline from framebuffer to display device:
+        ValidateBitsPlusImaging(win, 0, devname, 2);
     end
-    
+
     % Reset validation flag after first run:
     validated = 0;
 
     % Set colorConversionMode for this window to safe "undefined" default:
     colorConversionModeWin(win) = -1;
-    
+
     % Ready!
     return;
 end
@@ -1289,7 +1300,7 @@ if strcmpi(cmd, 'OpenWindowMono++') || strcmpi(cmd, 'OpenWindowMono++WithOverlay
 
     if (havespoken == 0) && (bitand(imagingmode, kPsychNeed32BPCFloat) || bitand(imagingmode, kPsychUse32BPCFloatAsap))
         fprintf('PTB - Info: Your framebuffer is configured for maximum precision. All internal processing will be done\n');
-        fprintf('PTB - Info: with about 23 bits of precision -- %s will be able to finally output with %i bits precision.\n', devname, devbits);
+        fprintf('PTB - Info: with about 23 bits of linear precision -- %s will be able to finally output with %i bits precision.\n', devname, devbits);
         if winfo.GLSupportsBlendingUpToBpc < 32
             fprintf('PTB - Info: Alpha-blending will not work at this precision with your hardware though.\n');
             fprintf('PTB - Info: You can either live with this limitation, or upgrade your graphics hardware to Direct3D-10\n');
@@ -1557,18 +1568,13 @@ if strcmpi(cmd, 'OpenWindowMono++') || strcmpi(cmd, 'OpenWindowMono++WithOverlay
         end
 
         if targetdevicetype == 1
-            % We need this weird evalin('base', ...); wrapper so the
-            % function gets called from the base-workspace, where the
-            % IMAGINGPIPE_GAMMATABLE variable is defined. We can only
-            % define it there reliably due to incompatibilities between
-            % Matlab and Octave in variable assignment inside Screen() :-(
-            rclutcmd = 'evalin(''base'', ''PsychDataPixx(1, IMAGINGPIPE_GAMMATABLE);'');';
+            rclutcmd = 'PsychDataPixx(1, IMAGINGPIPE_GAMMATABLE);';
             Screen('HookFunction', win, 'PrependMFunction', 'LeftFinalizerBlitChain', 'Upload new clut into DataPixx callback', rclutcmd);
         end
 
         Screen('HookFunction', win, 'Enable', 'LeftFinalizerBlitChain');
 
-        if (~isempty(stereomode) && stereomode == 1)
+        if ~isempty(stereomode) && ismember(stereomode, [1, 11])
             % This is only needed on quad-buffered stereo contexts for Bits+.
             % Enable CLUT updates via T-Lock on right stereo buffer as well:
 
@@ -1617,7 +1623,13 @@ if strcmpi(cmd, 'OpenWindowMono++') || strcmpi(cmd, 'OpenWindowMono++WithOverlay
     
     % Check validation:
     if ~validated
-        ValidateBitsPlusImaging(win, 0, devname);
+        % Validate imaging / display pipeline:
+
+        % From usercode drawing commands to framebuffer...
+        ValidateBitsPlusImaging(win, 0, devname, 1);
+
+        % ... and from framebuffer to display device:
+        ValidateBitsPlusImaging(win, 0, devname, 2);
     end
 
     % Reset validation flag after first run:
@@ -1671,24 +1683,175 @@ if strcmpi(cmd, 'CheckGPUSanity')
     return;
 end
 
+if strcmpi(cmd, 'UseFE1StereoGoggles')
+    % Assign onscreen window index:
+    if nargin < 2 || isempty(arg) || ~isa(arg, 'double') || Screen('WindowKind', arg) ~= 1
+        error('%s: "UseFE1StereoGoggles" called without valid onscreen window handle.', drivername);
+    end
+    win = arg;
+
+    winfo = Screen('GetWindowInfo', win);
+    if ~ismember(winfo.StereoMode, [1,11])
+        % No frame-sequential mode, no point in having sync lines -> No operation.
+        fprintf('UseFE1StereoGoggles: Info: Provided onscreen window is not switched to frame-sequential stereo mode. Call ignored.\n');
+        return;
+    end
+
+    if nargin < 3 || isempty(dummy) || ~ismember(dummy, [0, 1])
+        error('%s: "UseFE1StereoGoggles" called without valid enable flag.', drivername);
+    end
+    enableIt = dummy;
+
+    % Disable instead of enable requested?
+    if ~enableIt
+        % Existing hookslots found?
+        lslot = Screen('HookFunction', win, 'Query', 'LeftFinalizerBlitChain', 'RenderStereoFE1LeftTLock');
+        rslot = Screen('HookFunction', win, 'Query', 'RightFinalizerBlitChain', 'RenderStereoFE1RightTLock');
+        if (lslot ~= -1) && (rslot ~= -1)
+            % Yes. Delete them to disable the T-Lock blitting:
+            Screen('HookFunction', win, 'Remove', 'LeftFinalizerBlitChain' , lslot);
+            Screen('HookFunction', win, 'Remove', 'RightFinalizerBlitChain', rslot);
+        end
+
+        % Now the T-Lock blitting for frame-sequential stereo is off and
+        % the shutter goggles are in some unknown state. Set them to
+        % requested state, which defaults to 0 for "both eyes open":
+        if nargin < 4 || isempty(varargin{1}) || varargin{1} ~= 1
+            shutterState = 0;
+        else
+            shutterState = 1;
+        end
+
+        % Schedule T-Lock for setting final shutter state at next flip:
+        [encodedDIOdata, Mask, Data, Command] = bitsGoggles(shutterState, shutterState);
+        BitsPlusPlus('DIOCommand', win, 1, Mask, Data, Command);
+
+        % Execute!
+        Screen('Flip', win);
+
+        % Ok, shutter glasses should be open, we are done.
+        return;
+    end
+
+    % Enable case - Do the setup dance:
+
+    % Build display list for left-eye image T-Lock:
+    displist = glGenLists(1);
+    glNewList(displist, GL.COMPILE);
+
+    % Generate DIO T-Lock image as Matlab matrix to open the
+    % right shutter at the end of scanout of the left stereo image:
+    tlockdata = bitsGoggles(1, 0);
+
+    % Convert from Matlab matrix to OpenGL pixel format:
+    encodedDIOdata = uint8(zeros(3, 508));
+
+    % Pack 3 separate RGB planes into rows 1,2,3. As Matlabs data format is
+    % column major order, this will end up as tightly packed pixel array in
+    % format RGBRGBRGB.... just as glDrawPixels likes it.
+    encodedDIOdata(1,:) = tlockdata(1,:,1);
+    encodedDIOdata(2,:) = tlockdata(1,:,2);
+    encodedDIOdata(3,:) = tlockdata(1,:,3);
+
+    % Place T-Lock at start position (0, display height - 1):
+    [xDIO, yDIO] = Screen('WindowSize', win, 1);
+    yDIO = yDIO - 1;
+
+    % Add command sequence for this T-Lock code to display list:
+    glRasterPos2i(tlockXOffset, yDIO);
+    glDrawPixels(508, 1, GL.RGB, GL.UNSIGNED_BYTE, encodedDIOdata);
+
+    % Finish left display list;
+    glEndList;
+    pStringLeft = sprintf('glCallList(%i);', displist);
+
+    % Build display list for right-eye image T-Lock:
+    displist = glGenLists(1);
+    glNewList(displist, GL.COMPILE);
+
+    % Generate DIO T-Lock image as Matlab matrix to open the
+    % left shutter at the end of scanout of the right stereo image:
+    tlockdata = bitsGoggles(0, 1);
+
+    % Convert from Matlab matrix to OpenGL pixel format:
+    encodedDIOdata = uint8(zeros(3, 508));
+
+    % Pack 3 separate RGB planes into rows 1,2,3. As Matlabs data format is
+    % column major order, this will end up as tightly packed pixel array in
+    % format RGBRGBRGB.... just as glDrawPixels likes it.
+    encodedDIOdata(1,:) = tlockdata(1,:,1);
+    encodedDIOdata(2,:) = tlockdata(1,:,2);
+    encodedDIOdata(3,:) = tlockdata(1,:,3);
+
+    % Place T-Lock at start position (0, display height - 1):
+    [xDIO, yDIO] = Screen('WindowSize', win, 1);
+    yDIO = yDIO - 1;
+
+    % Add command sequence for this T-Lock code to display list:
+    glRasterPos2i(tlockXOffset, yDIO);
+    glDrawPixels(508, 1, GL.RGB, GL.UNSIGNED_BYTE, encodedDIOdata);
+
+    % Finish left display list;
+    glEndList;
+    pStringRight = sprintf('glCallList(%i);', displist);
+
+    % Ok 'win'dowhandle is fine. Any blue line sync function already applied?
+    lslot = Screen('HookFunction', win, 'Query', 'LeftFinalizerBlitChain', 'Builtin:RenderStereoSyncLine');
+    rslot = Screen('HookFunction', win, 'Query', 'RightFinalizerBlitChain', 'Builtin:RenderStereoSyncLine');
+
+    % Existing hookslots found?
+    if (lslot ~= -1) && (rslot ~= -1)
+        % Yes. Need to recreate with T-Lock for FE1 instead of standard blue-line sync lines:
+
+        % Destroy old ones:
+        Screen('HookFunction', win, 'Remove', 'LeftFinalizerBlitChain' , lslot);
+        Screen('HookFunction', win, 'Remove', 'RightFinalizerBlitChain', rslot);
+
+        % Insert new slots at former position of the old ones:
+        posstring = sprintf('InsertAt%iMFunction', lslot);
+        Screen('Hookfunction', win, posstring, 'LeftFinalizerBlitChain', 'RenderStereoFE1LeftTLock', pStringLeft);
+        posstring = sprintf('InsertAt%iMFunction', rslot);
+        Screen('Hookfunction', win, posstring, 'RightFinalizerBlitChain', 'RenderStereoFE1RightTLock', pStringRight);
+    else
+        % No. Create new slots at end:
+        Screen('Hookfunction', win, 'AppendMFunction', 'LeftFinalizerBlitChain', 'RenderStereoFE1LeftTLock', pStringLeft);
+        Screen('Hookfunction', win, 'AppendMFunction', 'RightFinalizerBlitChain', 'RenderStereoFE1RightTLock', pStringRight);
+    end
+
+    fprintf('UseFE1StereoGoggles: Info: Enabling automatic generation of sync signals for external FE1 shutter glasses.\n');
+
+    % Enable finalizer hookchains, if not already enabled:
+    Screen('HookFunction', win, 'Enable', 'LeftFinalizerBlitChain');
+    Screen('HookFunction', win, 'Enable', 'RightFinalizerBlitChain');
+
+    return;
+end
+
 error('%s: Unknown subcommand provided. Read "help BitsPlusPlus".', drivername);
 end
 
 % Helper function: Check if system already validated for current settings:
-function ValidateBitsPlusImaging(win, writefile, devname)
-    
+function ValidateBitsPlusImaging(win, writefile, devname, validationType)
+
     % Compute fingerprint of this system configuration:
     validated = 0;
     global GL;
-    
+
     screenid = Screen('WindowScreenNumber', win);
     [w, h] = Screen('WindowSize', screenid);
     d = Screen('PixelSize', win);
     v = Screen('Version');
     v = v.version;
+    c = Screen('Computer');
+    if IsLinux || IsOSX
+        c = c.kern.version;
+    else
+        c = c.system;
+    end
+
     gfxconfig = [ glGetString(GL.VENDOR) ':' glGetString(GL.RENDERER) ':' glGetString(GL.VERSION) ];
-    gfxconfig = sprintf('%s : Screen %i : Resolution %i x %i x %i : ScreenVersion = %s', gfxconfig, screenid, w, h, d, v);
-    
+    gfxconfig = sprintf('VTYPE %i : %s : Screen %i : Resolution %i x %i x %i : ScreenVersion = %s : System = %s', validationType, gfxconfig, screenid, w, h, d, v, c);
+
     if ~writefile
         % Check if a validation file exists and if it contains this
         % configuration:
@@ -1708,18 +1871,35 @@ function ValidateBitsPlusImaging(win, writefile, devname)
             fprintf('\n\n------------------------------------------------------------------------------------------------------------------\n')
             fprintf('\n\nThis specific configuration of graphics hardware, graphics driver and Psychtoolbox version has not yet been tested\n');
             fprintf('for correct working with %s for the given display screen, screen resolution and color depths.\n\n', devname);
-            fprintf('Please run the test script "BitsPlusImagingPipelineTest(%i);" once, so this configuration can be verified.\n', Screen('WindowScreenNumber', win));
+            if validationType == 1
+                fprintf('Please run the test script "BitsPlusImagingPipelineTest(%i);" once, so this configuration can be verified.\n', Screen('WindowScreenNumber', win));
+            else
+                if ~isempty(strfind(devname, 'Pixx'))
+                    fprintf('Please run the test script "BitsPlusIdentityClutTest(%i, 1);" once, so this configuration can be verified.\n', Screen('WindowScreenNumber', win));
+                    fprintf('Alternatively "BitsPlusIdentityClutTest(%i, 1, [], 1);" can be used for verification in L48 mode.\n', Screen('WindowScreenNumber', win));
+                else
+                    fprintf('Please run the test script "BitsPlusIdentityClutTest(%i);" once, so this configuration can be verified.\n', Screen('WindowScreenNumber', win));
+                    fprintf('Alternatively "BitsPlusIdentityClutTest(%i, 0, [], 1);" can be used for verification in Bits++ mode.\n', Screen('WindowScreenNumber', win));
+                end
+            end
             fprintf('After that test script suceeded, re-run your experiment script.\nThanks.\n');
             fprintf('\n');
             fprintf('Configuration to verify: %s\n', gfxconfig);
 
             RestoreCluts;
-            Screen('CloseAll'); ShowCursor; Priority(0);
-            
+            sca; ShowCursor; Priority(0);
+
+            % Perform device shutdown for display device:
+            if ~isempty(strfind(devname, 'Pixx'))
+                PsychDataPixx('ResetOnWindowClose');
+            else
+                BitsPlusPlus('ResetOnWindowClose');
+            end
+
             error('Configuration not yet verified. Please do it now.');
         end
     end
-    
+
     if writefile
         % Append current configuration to file to mark it as verified:
         [fid msg]= fopen([PsychtoolboxConfigDir 'ptbbitsplusplusvalidationfile.txt'], 'a');
@@ -1747,7 +1927,7 @@ function displist = SetupDIOFinalizer(win, stereomode)
     Screen('HookFunction', win, 'PrependMFunction', 'LeftFinalizerBlitChain', 'Render T-Lock DIO data callback', 'BitsPlusPlus(1);');
     Screen('HookFunction', win, 'Enable', 'LeftFinalizerBlitChain');
 
-    if (~isempty(stereomode) && stereomode == 1)
+    if ~isempty(stereomode) && ismember(stereomode, [1, 11])
         % This is only needed on quad-buffered stereo contexts.
         Screen('HookFunction', win, 'PrependMFunction', 'RightFinalizerBlitChain', 'Render T-Lock DIO data callback',  'BitsPlusPlus(1);');
         Screen('HookFunction', win, 'Enable', 'RightFinalizerBlitChain');
