@@ -64,6 +64,16 @@ function oldClut = LoadIdentityClut(windowPtr, loadOnNextFlip, lutType, disableD
 % 04/04/15   mk  Only use PsychGPUControl() dither disable on Windows by
 %                default. Only use on Linux + AMD as fallback if low-level
 %                dither disable fails.
+% 05/18/16   mk  Add detection for Broadcom VC4 SoC gpu in RaspberryPi.
+% 08/11/16   mk  Add detection based on winfo.DisplayCoreId on Linux to handle
+%                hybrid graphics laptops.
+% 06/05/17   mk  Add new lut handling for AMD gfx, to try to deal with pwl lut's
+%                in 10 bpc framebuffer mode on OSX. Preload some reasonable
+%                identity lut via LoadNormalizedGammaTable before using low-level
+%                identity lut setup, which can only deal with discrete lut's, not
+%                pwl lut's or such.
+% 07/10/17   MR  Added gfxhwtype=5 for AMD graphics cards under Windows (see also 
+%                the new variable 'ditherApiVer'). 
 
 global ptb_original_gfx_cluts;
 
@@ -112,7 +122,14 @@ if disableDithering && IsWin
     % we don't know how trustworthy it is. Also this only affects dithering,
     % not gamma table identity setup, so the code-pathes below must run anyway
     % for proper setup, even if their dithering disable effect may be redundant.
-    PsychGPUControl('SetDitheringEnabled', 0);
+    [rc, ditherApiVer] = PsychGPUControl('SetDitheringEnabled', 0);
+    % As long as there is no clear indication that it was dither API v2,
+    % we assume it was the old dithering API v1. Will be only used for AMD anyway.
+    if rc~=0 || isempty(ditherApiVer)
+        ditherApiVer = 1;
+    end
+else
+    ditherApiVer = 1;
 end
 
 % Ask Screen to use low-level setup code to configure the GPU for
@@ -121,6 +138,24 @@ end
 % is loaded, but should be the most reliable solution if it works:
 if ~IsWin
     % Low level control possible for some GPU's (e.g., AMD).
+
+    % For AMD hardware under OSX (and Linux, but doesn't matter yet), we preload identity
+    % lut of an ok quality via the conventional method, before using/trying the low-level
+    % setup. This in case low-level setup, which can currently only linearize the discrete
+    % hw lut's in 8 bpc format, fails to do a good job, because the display driver switches
+    % from the discrete hw lut to a piece-wise linear or non-linear lut. This seems to happen
+    % on some Apple Macs at least under macOS 10.12 when used in 10 bit native framebuffer mode.
+    % In such a case, although our low-level code does successfully load a 8 bpc, 256 slots
+    % discrete identity lut, that won't have the desired effect, as that discrete lut is not used.
+    % But if we upload a linearized lut conventionally, it should linearize the pwl/non-linear lut
+    % in the hardware. Note that for > 8 bit native modes we do not need a perfect 8 bpc pixel
+    % passthrough lut, as provided by our low-level setup code. A roughly linear lut will do to
+    % keep color/gamma calibration behaving as expected for 10 bpc display.
+    if strcmp(winfo.DisplayCoreId, 'AMD')
+      Screen('LoadNormalizedGammaTable', windowPtr, (linspace(0, 1, 256)' * ones(1, 3)), 0);
+      WaitSecs('YieldSecs', 0.25);
+    end
+
     % [] will enable full passthrough and force dithering off.
     % -1 will enable passthrough except for dithering, which is left at the OS default.
     if disableDithering
@@ -129,7 +164,7 @@ if ~IsWin
     else
         % Only identity LUTs, no color conversion, degamma etc., but leave the
         % dither settings untouched, so the OS + graphics driver stays in control:
-        passthroughrc = Screen('LoadNormalizedGammatable', windowPtr, -1);        
+        passthroughrc = Screen('LoadNormalizedGammatable', windowPtr, -1);
     end
 else
     passthroughrc = intmax;
@@ -150,7 +185,7 @@ else
     elseif IsOSX || IsLinux
         fprintf('LoadIdentityClut: Info: Could not use GPU low-level setup for setup of pixel passthrough. Will use fallback method.\n');
         % AMD GPU, aka GPU core of format 'R100', 'R500', ... starting with a 'R'?
-        if winfo.GPUCoreId(1) == 'R'
+        if ~isempty(strfind(winfo.DisplayCoreId, 'AMD'))
             % Some advice for AMD users on Linux and OSX:
             fprintf('LoadIdentityClut: Info: On your AMD/ATI GPU, you may get this working by loading the PsychtoolboxKernelDriver\n');
             fprintf('LoadIdentityClut: Info: on OS/X or using a Linux system properly setup with PsychLinuxConfiguration.\n');
@@ -210,16 +245,18 @@ else
         % systems:
 
         % Query OS type and version:
+        compinfo = Screen('Computer');
+
         if IsOSX
             % Which OS/X version?
-            compinfo = Screen('Computer');
             osxversion = sscanf(compinfo.system, '%*s %*s %i.%i.%i');
         end
 
         % We derive type of hardware and thereby our strategy from the vendor name:
         gfxhwtype = winfo.GLVendor;
 
-        if ~isempty(strfind(gfxhwtype, 'NVIDIA')) || ~isempty(strfind(gfxhwtype, 'nouveau'))
+        if ~isempty(strfind(winfo.DisplayCoreId, 'NVidia')) || (~IsLinux && (~isempty(strfind(gfxhwtype, 'NVIDIA')) || ...
+           ~isempty(strfind(gfxhwtype, 'nouveau'))))
             % NVidia card:
 
             % We start with assumption that it is a "normal" one:
@@ -230,7 +267,7 @@ else
                 % GeForce 250 on MS-Windows. Needs LUT type 1, according to Jon Peirce:
                 gfxhwtype = 1;
             end
-            
+
             % Is it a Geforce-8000 or later (G80 core or later) and is this OS/X?
             if ~isempty(strfind(winfo.GPUCoreId, 'G80')) && IsOSX
                 % 10.5.x Leopard?
@@ -247,7 +284,7 @@ else
                     % Yes. One of the releases with an embarassing amount of bugs,
                     % brought to you by Apple. Need to apply an especially ugly
                     % clut to force these cards into an identity mapping:
-                    
+
                     % Peter April reports his GeForce GT 330M on 10.6.8
                     % needs a type 0 LUT, so we handle this case:
                     if ~isempty(strfind(winfo.GLRenderer, '330'))
@@ -260,34 +297,48 @@ else
                     end
                 end
             end
-            
-            % Is this a latest generation NVidia QuadroFX card under Linux with NVidia binary blob?
-            [dummy, dummy2, nrslots] = Screen('ReadNormalizedGammatable', windowPtr); %#ok<ASGLU>
-            if IsLinux && ~isempty(strfind(winfo.GLRenderer, 'Quadro')) && (nrslots > 256)
-                % LUT type 3:
+
+            if IsLinux
+                % LUT type 3 seems to be right for both GeForce under nouveau-kms, and Quadro under
+                % nvidia blob, so probably for all NVidia gpus on Linux:
                 gfxhwtype = 3;
-                fprintf('LoadIdentityClut: NVidia Quadro or later on Linux with binary Blob detected. Enabling special type-III LUT.\n');
+                fprintf('LoadIdentityClut: NVidia gpu detected. Enabling type-III LUT.\n');
             end
         else
-            if ~isempty(strfind(gfxhwtype, 'ATI')) || ~isempty(strfind(gfxhwtype, 'AMD')) || ~isempty(strfind(gfxhwtype, 'Advanced Micro Devices')) || ...
-                    ~isempty(strfind(winfo.GLRenderer, 'DRI R')) || ~isempty(strfind(winfo.GLRenderer, 'on ATI R')) || ~isempty(strfind(winfo.GLRenderer, 'on AMD'))
+            if ~isempty(strfind(winfo.DisplayCoreId, 'AMD')) || (~IsLinux && (~isempty(strfind(gfxhwtype, 'ATI')) || ~isempty(strfind(gfxhwtype, 'AMD')) || ~isempty(strfind(gfxhwtype, 'Advanced Micro Devices')) || ...
+                    ~isempty(strfind(winfo.GLRenderer, 'DRI R')) || ~isempty(strfind(winfo.GLRenderer, 'on ATI R')) || ~isempty(strfind(winfo.GLRenderer, 'on AMD'))))
                 % AMD/ATI card:
 
                 % A good default at least on OS/X is type 1:
                 gfxhwtype = 1;
 
-                if (IsWin || IsLinux) && ~isempty(strfind(winfo.GPUCoreId, 'R600'))
+                if IsWin && ~isempty(strfind(winfo.GPUCoreId, 'R600'))
                     % At least the Radeon HD 3470 under Windows Vista and Linux needs type 0
                     % LUT's. Let's assume for the moment this is true for all R600
                     % cores, ie., all Radeon HD series cards.
-                    fprintf('LoadIdentityClut: ATI Radeon HD-2000 or later detected. Using type-0 LUT.\n');
-                    gfxhwtype = 0;
+                    % gfxhwtype==0 does not work for newer drivers. Since the driver version seems 
+                    % to be less indicative for the change, we base our decision on the dither API 
+                    % version implemented by AMD's display library (ADL). 
+                    if ditherApiVer==2
+                        fprintf('LoadIdentityClut: ATI Radeon HD-2000 or later with dither API v2 detected. Using type-5 LUT.\n');
+                        gfxhwtype = 5;
+                    else
+                        fprintf('LoadIdentityClut: ATI Radeon HD-2000 or later detected. Using type-0 LUT.\n');
+                        gfxhwtype = 0;
+                    end
                 elseif (IsLinux) && (~isempty(strfind(winfo.GLRenderer, 'DRI R')) || ~isempty(strfind(winfo.GLRenderer, 'on ATI R')) || ~isempty(strfind(winfo.GLRenderer, 'on AMD')))
-                    % At least the Radeon R3xx/4xx/5xx under Linux with DRI2 Mesa needs type 0
-                    % LUT's. Let's assume for the moment this is true for all R600
-                    % cores, ie., all Radeon HD series cards.
-                    fprintf('LoadIdentityClut: ATI Radeon on Linux DRI2 detected. Using type-0 LUT.\n');
-                    gfxhwtype = 0;
+                    if ~isempty(strfind(winfo.GPUCoreId, 'R600'))
+                        % At least the Radeon R9 380 Tonga Pro with DCE10 display engine under Linux with DRI2 Mesa needs type 3
+                        % LUT's. Let's assume for the moment this is true for all such R600+ cores, ie., all Radeon HD series cards.
+                        fprintf('LoadIdentityClut: ATI Radeon HD-2000 or later on Linux DRI2/DRI3 detected. Using type-3 LUT.\n');
+                        gfxhwtype = 3;
+                    else
+                        % At least the Radeon R3xx/4xx/5xx under Linux with DRI2 Mesa needs type 0
+                        % LUT's. Let's assume for the moment this is true for all R600
+                        % cores, ie., all Radeon HD series cards.
+                        fprintf('LoadIdentityClut: ATI Radeon on Linux DRI2 detected. Using type-0 LUT.\n');
+                        gfxhwtype = 0;
+                    end
                 elseif IsOSX && (~isempty(strfind(winfo.GLRenderer, 'Radeon HD 5')))
                     % At least on OS/X 10.6 with ATI Radeon HD-5000 series,
                     % type 2 seems to be the correct choice, according to Cesar
@@ -295,10 +346,33 @@ else
                     fprintf('LoadIdentityClut: ATI Radeon HD-5000 Evergreen on OS/X detected. Using type-2 LUT.\n');
                     gfxhwtype = 2;
                 end
-            elseif ~isempty(strfind(gfxhwtype, 'Intel'))
-                % Intel card: Type 0 LUT is correct at least on Linux:
+            elseif ~isempty(strfind(winfo.DisplayCoreId, 'Intel')) || (~IsLinux && (~isempty(strfind(gfxhwtype, 'Intel'))))
+                % Intel card: Type 0 LUT is correct at least on Linux versions
+                % < Linux 4.7. Take this as a baseline:
                 gfxhwtype = 0;
-                fprintf('LoadIdentityClut: Intel integrated graphics chip detected. Using type-0 LUT.\n');
+
+                % Linux got a new color management for Intel-kms in 4.7+
+                if IsLinux
+                    osrelease = sscanf(compinfo.kern.osrelease, '%i.%i');
+                    if (osrelease(1) > 4) || (osrelease(1) == 4 && osrelease(2) >= 7)
+                        % Linux 4.7 or later. New color management for Intel:
+                        gfxhwtype = 4;
+                    else
+                        % Older kernel: Old color management for Intel:
+                        gfxhwtype = 0;
+                    end
+                end
+
+                if IsOSX
+                    % Type 2 needed at least on 10.12.1 with at least Intel HD-4000:
+                    gfxhwtype = 2;
+                end
+
+                fprintf('LoadIdentityClut: Intel integrated graphics chip detected. Using type-%i LUT.\n', gfxhwtype);
+            elseif ~isempty(strfind(gfxhwtype, 'Broadcom')) && ~isempty(strfind(winfo.GLRenderer, 'VC4'))
+                % VC4 in RaspberryPi: Type 0 LUT is correct at least on Linux:
+                gfxhwtype = 0;
+                fprintf('LoadIdentityClut: VideoCore-4 SoC graphics chip detected. Using type-0 LUT.\n');
             else
                 % Unknown card: Default to NVidia behaviour:
                 gfxhwtype = 0;
@@ -386,7 +460,26 @@ else
         oldClut = Screen('LoadNormalizedGammaTable', windowPtr, loadlut, loadOnNextFlip);
     end
 
-    if ~ismember(gfxhwtype, [-1,0,1,2,3])
+    if gfxhwtype == 4
+        % This is a very close approximation of the default gamma table used by the
+        % Intel drm/kms display driver for Linux 4.8 and later, after they implemented
+        % the new color management. At least valid for Intel HD-4000 IvyBridge down to
+        % a match which is accurate to approx. 16 bits.
+        oldClut = Screen('LoadNormalizedGammaTable', windowPtr, (0:1/256:0.99611)' * ones(1, 3), loadOnNextFlip);
+    end
+
+    if gfxhwtype == 5
+        % Probably all AMD Radeon cards under Windows with graphics card drivers 
+        % Catalyst 15.7.1 or later (a better indicator than the driver version is 
+        % the version of the ADL dither API though, which changed in an incompatible  
+        % way sometime around 15.7.1). This is equivalent to gfxhwtype==4 but with 
+        % the last element set to 1.
+        % Tested with HD7750 and RX460 under Win7/Win10. 
+        % See also https://github.com/Psychtoolbox-3/Psychtoolbox-3/issues/402.
+        oldClut = Screen('LoadNormalizedGammaTable', windowPtr, [(0:254)*256/(2^16-1), 1]' * ones(1, 3), loadOnNextFlip);        
+    end    
+
+    if ~ismember(gfxhwtype, [-1,0,1,2,3,4,5])
         sca;
         error('Could not upload identity CLUT to GPU! Invalid LUT or invalid LUT id or other invalid arguments passed?!?');
     end
