@@ -15,12 +15,10 @@ function PsychtoolboxPostInstallRoutine(isUpdate, flavor)
 % Currently the routine performs the following tasks:
 %
 % 1. Clean up the Matlab/Octave path to Psychtoolbox: Remove unneeded .svn subfolders.
-% 2. Contact the Psychtoolbox server to perform online registration of this
-%    working copy of Psychtoolbox.
-% 3. Add the PsychJava subfolder to the static Matlab class-path if neccessary.
+% 2. Add the PsychJava subfolder to the static Matlab class-path if neccessary.
 %    This enables the Java-based GetChar support on Matlab.
-% 4. Add the PsychStartup.m routine to Matlab's startup.m file on Windows.
-% 5. Perform post-installation checks and basic troubleshooting.
+% 3. Add the PsychStartup.m routine to Matlab's startup.m file on Windows.
+% 4. Perform post-installation checks and basic troubleshooting.
 
 %
 % History:
@@ -69,6 +67,10 @@ function PsychtoolboxPostInstallRoutine(isUpdate, flavor)
 %            64-Bit Octave-3 support for OSX removed. (MK)
 %            OSX 10.8 and 10.9 support removed. (MK)
 % 07/06/2016 No upstream support for 32-Bit Octave on Linux anymore. NeuroDebian provides it though. (MK)
+% 12/26/2016 No support for OSX 10.10 anymore. No support for Octave-4.0 on non-Linux,
+%            support 64-Bit Octave 4.2 on Windows and OSX instead. (MK)
+% 01/03/2017 Fix Matlab incompatibility with __octave_config_info__. (MK)
+% 04/07/2018 Remove PsychtoolboxRegistration for now. (MK)
 
 fprintf('\n\nRunning post-install routine...\n\n');
 
@@ -246,14 +248,14 @@ if IsOSX
         minorver = inf;
     end
 
-    % Is the operating system version < 10.10?
-    if minorver < 10
-        % Yes. This is MacOSX 10.9 or earlier, i.e., older than 10.10
-        % Yosemite. PTB will not work on such an old system:
+    % Is the operating system version < 10.11?
+    if minorver < 11
+        % Yes. This is MacOSX 10.10 or earlier, i.e., older than 10.11
+        % El Capitan. PTB will not work on such an old system:
         fprintf('\n\n\n\n\n\n\n\n==== WARNING WARNING WARNING WARNING ====\n\n');
         fprintf('Your operating system is Mac OS/X version 10.%i.\n\n', minorver);
         fprintf('This release of Psychtoolbox-3 is not compatible\n');
-        fprintf('to OSX versions older than 10.10 "Yosemite".\n');
+        fprintf('to OSX versions older than 10.11 "El Capitan".\n');
         fprintf('That means that almost all functionality will not work!\n\n');
         fprintf('You could download an older version of Psychtoolbox\n');
         fprintf('onto your system to get better results. See our Wiki for help.\n');
@@ -390,15 +392,15 @@ if IsOctave
         fprintf('=====================================================================\n\n');
     end
 
-    if (~IsLinux && (octavemajorv < 4)) || (octavemajorv < 3) || (octavemajorv == 3 && octaveminorv < 2)
+    if (~IsLinux && (octavemajorv ~= 4 || octaveminorv ~= 2)) || (octavemajorv < 3) || (octavemajorv == 3 && octaveminorv < 8)
         fprintf('\n\n=================================================================================\n');
-        fprintf('WARNING: Your version %s of Octave is obsolete. We strongly recommend\n', version);
+        fprintf('WARNING: Your version %s of Octave is incompatible with this release. We strongly recommend\n', version);
         if IsLinux
-            % On Linux everything >= 3.2 is fine:
-            fprintf('WARNING: using the latest stable version of the Octave 3.2.x series or later for use with Psychtoolbox.\n');
+            % On Linux everything >= 3.8 is fine:
+            fprintf('WARNING: using the latest stable version of the Octave 3.8 or 4.0 series for use with Psychtoolbox.\n');
         else
-            % On Windows/OSX we only care about >= 4.0 atm:
-            fprintf('WARNING: using the latest stable version of the Octave 4.0.x series for use with Psychtoolbox.\n');
+            % On Windows/OSX we only care about 4.2 atm:
+            fprintf('WARNING: using the latest stable version of the Octave 4.2.x series for use with Psychtoolbox.\n');
         end
         fprintf('WARNING: Stuff may not work at all or only suboptimal with other versions and we\n');
         fprintf('WARNING: don''t provide any support for such old versions.\n');
@@ -411,17 +413,17 @@ if IsOctave
         % Need to copy the Octave runtime libraries somewhere our mex files can find them. The only low-maintenance
         % way of dealing with this mess of custom library pathes per octave version, revision and packaging format.
         % Preferred location is the folder with our mex files - found by rpath = @loader_path
-        if ~copyfile([octave_config_info.octlibdir filesep 'liboctinterp.3.dylib'], [rdir filesep], 'f') || ...
-           ~copyfile([octave_config_info.octlibdir filesep 'liboctave.3.dylib'], [rdir filesep], 'f')
+        if ~copyfile([GetOctlibDir filesep 'liboctinterp.4.dylib'], [rdir filesep], 'f') || ...
+           ~copyfile([GetOctlibDir filesep 'liboctave.4.dylib'], [rdir filesep], 'f')
             % Copy into our mex files folder failed. A second location where the linker will search is the
             % $HOME/lib directory of the current user, so try that as target location:
             tdir = PsychHomeDir('lib');
             fprintf('\n\nFailed to copy Octave runtime libraries to mex file folder [%s].\nRetrying in users private lib dir: %s ...\n', rdir, tdir);
-            if ~copyfile([octave_config_info.octlibdir filesep 'liboctinterp.3.dylib'], tdir, 'f') || ...
-               ~copyfile([octave_config_info.octlibdir filesep 'liboctave.3.dylib'], tdir, 'f')
+            if ~copyfile([GetOctlibDir filesep 'liboctinterp.4.dylib'], tdir, 'f') || ...
+               ~copyfile([GetOctlibDir filesep 'liboctave.4.dylib'], tdir, 'f')
                 fprintf('\nFailed to copy runtime libs to [%s] as well :(.\n', tdir);
                 fprintf('Our mex files will likely not work this way. Maybe the directories lack file write permissions?\n');
-                fprintf('\n\n\nA last workaround would be to restart octave from a terminal via this line:\n\nexport DYLD_LIBRARY_PATH=%s ; octave\n\n\n', octave_config_info.octlibdir);
+                fprintf('\n\n\nA last workaround would be to restart octave from a terminal via this line:\n\nexport DYLD_LIBRARY_PATH=%s ; octave\n\n\n', GetOctlibDir);
             end
         end
     end
@@ -534,7 +536,7 @@ if IsWin && ~IsOctave
         end
         fprintf('ERROR: You must execute that installer as an administrator user. Exit Matlab before the installation, then restart it.\n');
         fprintf('ERROR: After fixing the problem, restart this installation/update routine.\n\n');
-        fprintf('ERROR: You can also just do a: cd(PsychtoolboxRoot); SetupPsychtoolbox; PsychtoolboxRegistration(%i, ''%s'');\n\n', isUpdate, flavor);
+        fprintf('ERROR: You can also just do a: cd(PsychtoolboxRoot); SetupPsychtoolbox;\n\n');
         fprintf('ERROR: This will avoid a full download of Psychtoolbox over the internet and just finish the setup.\n');
         
         fprintf('\n\nInstallation aborted. Fix the reported problem and retry.\n\n');
@@ -618,27 +620,18 @@ try
         PsychGPUControl('FullScreenWindowDisablesCompositor', 1);
     end
 
-    % Try to execute online registration routine: This should be fail-safe in case
-    % of no network connection.
-    fprintf('\n\n');
-    PsychtoolboxRegistration(isUpdate, flavor);
-    fprintf('\n\n\n');
-
     % Tell user we're successfully done:
     fprintf('\nDone with post-installation. Psychtoolbox is ready for use.\n\n\n');
 
 catch
     fprintf('\n\n');
-    fprintf('Screen() or online registration failed to work for some reason:\n\n');
+    fprintf('Screen() failed to work for some reason:\n\n');
     fprintf('Check the troubleshooting instructions on our Wiki (Download section \n');
     fprintf('and FAQ section, maybe also the Bugs section).\n\n');
     fprintf('Once you manage to fix the problem (simply type ''AssertOpenGL'' to verify\n');
     fprintf('that stuff works now), you do not need to run the installation routine again,\n');
     fprintf('but can start working immediately.\n\n');
-    fprintf('However, we kindly ask you to execute the following command after everything works,\n');
-    fprintf('so your copy gets registered by us for statistical purpose:\n\n');
-    fprintf('PsychtoolboxRegistration(%i, ''%s'');\n\n', isUpdate, flavor);
-    fprintf('Thanks! Press RETURN or ENTER to confirm you read and understood the above message.\n');
+    fprintf('Press RETURN or ENTER to confirm you read and understood the above message.\n');
     pause;
     fprintf('\n\n');
 end
@@ -709,7 +702,6 @@ fprintf('\nEnjoy!\n\n');
 fprintf('Press RETURN or ENTER to confirm you read and understood the above message.\n');
 pause;
 fprintf('\n\n');
-PTBSurvey;
 
 % Clear out everything:
 if IsWin
